--
-- ChangeTransportMode
--
-- @package 	fs17.saphir_tl_180.specializations.ChangeTransportMode
-- @auhtor 		IceBlade <www.cg-production.com>
-- @copyright 	2018 CG-Production
-- @license    	www.cg-production.com/license/FS_Script_Licence_2018_1.txt
-- @version		1.0.0.2
-- @history		<1.0.0.0> <25.02.2018> creation
-- 

ChangeTransportMode = {};

function ChangeTransportMode.prerequisitesPresent(specializations)
	return true;
end;

function ChangeTransportMode:preLoad()
	self.getCanTipToGround  = Utils.overwrittenFunction(self.getCanTipToGround , ChangeTransportMode.getCanTipToGround);
	self.getl10n = ChangeTransportMode.getl10n;
	self.changeTransportMode = ChangeTransportMode.changeTransportMode;
	self.changeBackDoor = ChangeTransportMode.changeBackDoor;
end;

function ChangeTransportMode:load()

	-- key
	local key = "vehicle.transportModes";

	-- load xml attributes
	self.transportModes = {}

	self.transportModes[1] = {};
	self.transportModes[1].fillVolumeIndex = getXMLInt(self.xmlFile, key .. ".fillable#fillVolumeIndex");

	self.transportModes[2] = {};
	self.transportModes[2].backDoor = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key .. '.objects#backDoor'));
	self.transportModes[2].backDoorCol = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key .. '.objects#backDoorCol'));

	self.tipReferencePointBackup = self.currentTipReferencePointIndex;
	self.currentTransportMode = 1;
	self.lastTransportMode = 1;
	self.isBackDoorActiveLast = true;
	self.isBackDoorActive = true;
end;

function ChangeTransportMode:postLoad(savegame)
	if savegame ~= nil and not savegame.resetVehicles then
		local loadCurrentTransportMode = getXMLInt(savegame.xmlFile, savegame.key .. "#currentTransportMode");
		if loadCurrentTransportMode ~= nil then
			self:changeTransportMode(loadCurrentTransportMode, true);
		end;

		local loadBackDoorActive = getXMLBool(savegame.xmlFile, savegame.key .. "#backDoorActive");
		if loadBackDoorActive ~= nil then
			if self.currentTransportMode == 2 and not loadBackDoorActive then
				self:changeBackDoor(false, true);
			end;
		end;
	end;
end;

function ChangeTransportMode:getSaveAttributesAndNodes(nodeIdent)
	local transportMode = ('currentTransportMode="%s"'):format(self.currentTransportMode);
	local backDoor = ('backDoorActive="%s"'):format(self.isBackDoorActive);
	return transportMode .. " " .. backDoor;
end;

function ChangeTransportMode:delete()
end;

function ChangeTransportMode:readStream(streamId, connection)
	local transportMode = streamReadUInt8(streamId);
	local backDoorMode = streamReadBool(streamId);
	if transportMode ~= nil then
		self:changeTransportMode(transportMode, true);
	end;
	if backDoorMode ~= nil then
		self:changeBackDoor(backDoorMode, true);
	end;
end;

function ChangeTransportMode:writeStream(streamId, connection)
	streamWriteUInt8(streamId, self.currentTransportMode);
	streamWriteBool(streamId, self.isBackDoorActive);
end;

function ChangeTransportMode:mouseEvent(posX, posY, isDown, isUp, button)
end;

function ChangeTransportMode:keyEvent(unicode, sym, modifier, isDown)
end;

function ChangeTransportMode:update(dt)
	if self:getIsActiveForInput() then
		if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA4) then
			if self.currentTransportMode == 1 then
				if self:getFillLevel() == 0 then
					self:changeTransportMode(2, false);
				else
					g_currentMission:showBlinkingWarning(g_i18n:getText("transportMode_error_isFilled"));
				end;
			else
				if not self.areBeltsFasten then
					self:changeTransportMode(1, false);
				end;
			end;
		end;

		if self.currentTransportMode == 2 then
			if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
				if not self.isBackDoorActive then
					self:changeBackDoor(true, false);
				else
					self:changeBackDoor(false, false);
				end;
			end;
		end;
	end;

	if self.isBackDoorActive ~= self.isBackDoorActiveLast then
		if self.isBackDoorActive then
			setVisibility(self.transportModes[2].backDoor, true);
			setCollisionMask(self.transportModes[2].backDoorCol, 8194);
		else
			setVisibility(self.transportModes[2].backDoor, false);
			setCollisionMask(self.transportModes[2].backDoorCol, 0);
		end;
		self.isBackDoorActiveLast = self.isBackDoorActive;
	end;

	if self.currentTransportMode ~= self.lastTransportMode then
		if self.currentTransportMode == 1 then
			self:changeBackDoor(true, false);
			self.allowFillFromAir = true;
			self.currentTipReferencePointIndex = self.tipReferencePointBackup;
		else
			self.allowFillFromAir = false;
			self.currentTipReferencePointIndex = nil;
		end;
		self.lastTransportMode = self.currentTransportMode;
	end;
end;

function ChangeTransportMode:updateTick(dt)
end;

function ChangeTransportMode:draw()
	g_currentMission:addHelpButtonText(self:getl10n(), InputBinding.IMPLEMENT_EXTRA4, nil, GS_PRIO_HIGH);
	if self.currentTransportMode == 2 then
		if not getVisibility(self.transportModes[2].backDoor) then
			g_currentMission:addHelpButtonText(g_i18n:getText("transportMode_addDoor"), InputBinding.IMPLEMENT_EXTRA2, nil, GS_PRIO_HIGH);
		else
			g_currentMission:addHelpButtonText(g_i18n:getText("transportMode_removeDoor"), InputBinding.IMPLEMENT_EXTRA2, nil, GS_PRIO_HIGH);
		end;
	end;
end;

function ChangeTransportMode:getl10n()
	local localizationKey = "transportMode_" .. self.currentTransportMode;
	local text = g_i18n:getText("changeTransportMode") .. " " .. g_i18n:getText(localizationKey);
	return text;
end;

function ChangeTransportMode:getCanTipToGround()
	local canTipToGround = false;

	if self.currentTransportMode == 1 then
		canTipToGround = true;
	end;

	if not self.areBeltsFasten then
		canTipToGround = true;
	end;

	if canTipToGround then
	    if TipUtil.isValid() then
	        local currentFillType = self:getUnitFillType(self.trailer.fillUnitIndex);
	        if currentFillType == FillUtil.FILLTYPE_UNKNOWN then
	            return true;
	        end
	        return TipUtil.getCanTipToGround(currentFillType);
	    else
	        return false;
	    end;
	else
		return false;
	end;
end;

function ChangeTransportMode:changeTransportMode(mode, noEventSend)
	if mode ~= nil then
		self.currentTransportMode = mode;
		ChangeTransportModeEvent.sendEvent(self, self.currentTransportMode, self.isBackDoorActive, noEventSend);
	end;
	return nil;
end;

function ChangeTransportMode:changeBackDoor(activity, noEventSend)
	if activity ~= nil then
		self.isBackDoorActive = activity;
		ChangeTransportModeEvent.sendEvent(self, self.currentTransportMode, self.isBackDoorActive, noEventSend);
	end;
	return nil;
end;