--
-- ChangeTransportMode
--
-- @package 	fs17.saphir_tl_180.specializations.ChangeTransportMode
-- @auhtor 		IceBlade <www.cg-production.com>
-- @copyright 	2018 CG-Production
-- @license    	www.cg-production.com/license/FS_Script_Licence_2018_1.txt
-- @version		1.0.0.2
-- @history		<1.0.0.0> <25.02.2018> creation
-- 

ChangeTransportModeEvent = {}
ChangeTransportModeEvent_mt = Class(ChangeTransportModeEvent, Event)
InitEventClass(ChangeTransportModeEvent, 'ChangeTransportModeEvent')

function ChangeTransportModeEvent:emptyNew()
    local self = Event:new(ChangeTransportModeEvent_mt);
    return self;
end

function ChangeTransportModeEvent:new(vehicle, mode, backDoorMode)
    local self = ChangeTransportModeEvent:emptyNew();
    self.vehicle = vehicle;
    self.mode = mode;
    self.backDoorMode = backDoorMode;
    return self;
end

function ChangeTransportModeEvent:writeStream(streamId, connection)
    writeNetworkNodeObject(streamId, self.vehicle);
    streamWriteUInt8(streamId, self.mode);
    streamWriteBool(streamId, self.backDoorMode);
end

function ChangeTransportModeEvent:readStream(streamId, connection)
    self.vehicle = readNetworkNodeObject(streamId);
    self.mode = streamReadUInt8(streamId);
    self.backDoorMode = streamReadBool(streamId);
    self:run(connection);
end

function ChangeTransportModeEvent:run(connection)
    if not connection:getIsServer() then
        g_server:broadcastEvent(self, nil, connection, self.vehicle, self.mode, self.backDoorMode);
    end;
    self.vehicle:changeTransportMode(self.mode, true);
    self.vehicle:changeBackDoor(self.backDoorMode, true);
end

function ChangeTransportModeEvent.sendEvent(vehicle, mode, backDoorMode, noEventSend)
    if noEventSend == nil or noEventSend == false then
        if g_server ~= nil then
            g_server:broadcastEvent(ChangeTransportModeEvent:new(vehicle, mode, backDoorMode), nil, nil, vehicle);
        else
            g_client:getServerConnection():sendEvent(ChangeTransportModeEvent:new(vehicle, mode, backDoorMode));
        end;
    end
end;